from StringIO import StringIO
from Products.Archetypes.Extensions.utils import install_subskin
from Products.CMFCore.utils import getToolByName
from Products.ExternalMethod.ExternalMethod import manage_addExternalMethod
from Acquisition import aq_base

from Products.Mail2Image.config import *

externalmethods = ('external_mail2image', )

## portal properties
portalproperties = \
(
    {
        'id'    : 'fonts',
        'value' : DEFAULT_FONTS,
        'type'  : 'lines'
    },
    {
        'id'    : 'to_lowercase',
        'value' : True,
        'type'  : 'boolean'
    },
    {
        'id'    : 'encoding',
        'value' : DEFAULT_ENCODING,
        'type'  : 'lines'
    },
    {
        'id'    : 'seed',
        'value' : 'changethistosometext',
        'type'  : 'string'
    },
    {
        'id'    : 'link_contactform',
        'value' : False,
        'type'  : 'boolean'
    },
    {
        'id'    : 'link_whitelist',
        'value' : ('@domain.de',),
        'type'  : 'lines'
    },
    {
        'id'    : 'link_blacklist',
        'value' : (),
        'type'  : 'lines'
    },
    {
        'id'    : 'enable_captcha',
        'value' : True,
        'type'  : 'boolean'
    },
    {
        'id'    : 'subject_prefix',
        'value' : '',
        'type'  : 'string'
    },
    {
        'id'    : 'message_template',
        'value' : '%(message)s',
        'type'  : 'text'
    },
)       

## add external methods in custom folder!
def addExternalMethods(self,out):
    portalskin=getToolByName(self,'portal_skins')
    if not portalskin:
        return
    for scripts in externalmethods:
        try: 
            manage_addExternalMethod(portalskin, 
                                     scripts, 
                                     scripts, 
                                     PROJECTNAME+'.'+scripts, 
                                     scripts)
            out.write('Adding external method %s\n' % scripts)
        except:
            out.write('Warning: Couldn\'t add external method %s\n' % scripts)
        


## remove external methods
def removeExternalMethods(self,out):
    portalskin=getToolByName(self,'portal_skins')
    if not portalskin:
        return
    for scripts in externalmethods:
        if not hasattr(aq_base(portalskin), scripts):
            continue
        try:
            portalskin.manage_delObjects(ids=[scripts,])
            out.write('Removing external method %s\n' % scripts)
        except:
            out.write('Warning: Couldn\'t remove external method %s\n' %
                      scripts)

## add portal properties
def addPortalProperties(self,out):
    portal_props = getToolByName(self, 'portal_properties')
    if not hasattr(portal_props, 'mail2image_properties'):
        portal_props.addPropertySheet('mail2image_properties',
                                      'Mail2Image properties')
    props = getattr(portal_props, 'mail2image_properties')
    for property in portalproperties:
        if hasattr(props, property['id']):
            continue
        try:
            props._setProperty(property['id'],
                               property['value'],
                               property['type'])
            out.write('Adding portal property %s\n' % property['id'])
        except:
            out.write('Warning: Couldn\'t add portal porperty %s\n' %
                      property['id'])
                                      
def addConfiglet(self, out):
    try:
        portal_conf=getToolByName(self,'portal_controlpanel')
    except AttributeError:
        print >>out, "Configlet could not be installed"
        return
    try:
        portal_conf.registerConfiglet(
            'mail2image', 
            PROJECTNAME,
            'string:${portal_url}/mail2image_config',
            '',                 # a condition
            'Manage portal',    # access permission
            'Products',         # section
                                #(Plone,Products,Members)
            1,                  # visibility
            PROJECTNAME,
            'mail_icon.gif',    # icon in control_panel
            'Mail2Image Tool',
            None )
    except KeyError:
        pass # Get KeyError when registering duplicate configlet.

def removeConfiglet(self, out):
    configTool = getToolByName(self, 'portal_controlpanel', None)
    if configTool:
        configTool.unregisterConfiglet('mail2image')
        out.write('Removed Mail2Image configlet\n')

## install
def install(self):
    out = StringIO()
    install_subskin(self, out, GLOBALS)
    addExternalMethods(self, out)
    addPortalProperties(self, out)
    addConfiglet(self, out)
    print >> out, "Successfully installed %s." % PROJECTNAME
    return out.getvalue() 

## uninstall
def uninstall(self):
    out=StringIO()
    removeExternalMethods(self, out)
    removeConfiglet(self, out)
    return out.getvalue()
